@@ -1067,7 +1067,7 @@ fn show_statistics_syntax_errors() {
     success: false
     exit_code: 1
     ----- stdout -----
-    1		syntax-error
+    1		invalid-syntax
     Found 1 error.
 
     ----- stderr -----
@@ -1080,7 +1080,7 @@ fn show_statistics_syntax_errors() {
     success: false
     exit_code: 1
     ----- stdout -----
-    1		syntax-error
+    1		invalid-syntax
     Found 1 error.
 
     ----- stderr -----
@@ -1093,7 +1093,7 @@ fn show_statistics_syntax_errors() {
     success: false
     exit_code: 1
     ----- stdout -----
-    1		syntax-error
+    1		invalid-syntax
     Found 1 error.
 
     ----- stderr -----
