@@ -1,9 +1,9 @@
 use std::borrow::Cow;
 
-use ruff_formatter::{format_args, write, FormatError, FormatOptions, SourceCode};
-use ruff_python_ast::{AnyNodeRef, AstNode, NodeKind, PySourceType};
+use ruff_formatter::{FormatError, FormatOptions, SourceCode, format_args, write};
+use ruff_python_ast::{AnyNodeRef, NodeKind, PySourceType};
 use ruff_python_trivia::{
-    is_pragma_comment, lines_after, lines_after_ignoring_trivia, lines_before, CommentLinePosition,
+    CommentLinePosition, is_pragma_comment, lines_after, lines_after_ignoring_trivia, lines_before,
 };
 use ruff_text_size::{Ranged, TextLen, TextRange};
 
@@ -13,11 +13,11 @@ use crate::prelude::*;
 use crate::statement::suite::should_insert_blank_line_after_class_in_stub_file;
 
 /// Formats the leading comments of a node.
-pub(crate) fn leading_node_comments<T>(node: &T) -> FormatLeadingComments
+pub(crate) fn leading_node_comments<'a, T>(node: T) -> FormatLeadingComments<'a>
 where
-    T: AstNode,
+    T: Into<AnyNodeRef<'a>>,
 {
-    FormatLeadingComments::Node(node.as_any_node_ref())
+    FormatLeadingComments::Node(node.into())
 }
 
 /// Formats the passed comments as leading comments
@@ -85,7 +85,7 @@ pub(crate) struct FormatLeadingAlternateBranchComments<'a> {
 
 impl Format<PyFormatContext<'_>> for FormatLeadingAlternateBranchComments<'_> {
     fn fmt(&self, f: &mut PyFormatter) -> FormatResult<()> {
-        if self.last_node.map_or(false, |preceding| {
+        if self.last_node.is_some_and(|preceding| {
             should_insert_blank_line_after_class_in_stub_file(preceding, None, f.context())
         }) {
             write!(f, [empty_line(), leading_comments(self.comments)])?;
@@ -164,7 +164,7 @@ impl Format<PyFormatContext<'_>> for FormatTrailingComments<'_> {
                         line_suffix(
                             &format_args![
                                 empty_lines(lines_before_comment),
-                                format_comment(trailing)
+                                format_comment(trailing),
                             ],
                             // Reserving width isn't necessary because we don't split
                             // comments and the empty lines expand any enclosing group.
@@ -192,11 +192,11 @@ impl Format<PyFormatContext<'_>> for FormatTrailingComments<'_> {
 }
 
 /// Formats the dangling comments of `node`.
-pub(crate) fn dangling_node_comments<T>(node: &T) -> FormatDanglingComments
+pub(crate) fn dangling_node_comments<'a, T>(node: T) -> FormatDanglingComments<'a>
 where
-    T: AstNode,
+    T: Into<AnyNodeRef<'a>>,
 {
-    FormatDanglingComments::Node(node.as_any_node_ref())
+    FormatDanglingComments::Node(node.into())
 }
 
 pub(crate) fn dangling_comments(comments: &[SourceComment]) -> FormatDanglingComments {
@@ -381,14 +381,13 @@ impl Format<PyFormatContext<'_>> for FormatTrailingEndOfLineComment<'_> {
             0
         } else {
             // Start with 2 because of the two leading spaces.
-            let width = 2u32.saturating_add(
+
+            2u32.saturating_add(
                 TextWidth::from_text(&normalized_comment, f.options().indent_width())
                     .width()
                     .expect("Expected comment not to contain any newlines")
                     .value(),
-            );
-
-            width
+            )
         };
 
         write!(
@@ -535,22 +534,13 @@ fn strip_comment_prefix(comment_text: &str) -> FormatResult<&str> {
 /// ```
 ///
 /// This builder will insert a single empty line before the comment.
-pub(crate) fn empty_lines_before_trailing_comments<'a>(
-    f: &PyFormatter,
-    comments: &'a [SourceComment],
+pub(crate) fn empty_lines_before_trailing_comments(
+    comments: &[SourceComment],
     node_kind: NodeKind,
-) -> FormatEmptyLinesBeforeTrailingComments<'a> {
-    // Black has different rules for stub vs. non-stub and top level vs. indented
-    let empty_lines = match (f.options().source_type(), f.context().node_level()) {
-        (PySourceType::Stub, NodeLevel::TopLevel(_)) => 1,
-        (PySourceType::Stub, _) => u32::from(node_kind == NodeKind::StmtClassDef),
-        (_, NodeLevel::TopLevel(_)) => 2,
-        (_, _) => 1,
-    };
-
+) -> FormatEmptyLinesBeforeTrailingComments {
     FormatEmptyLinesBeforeTrailingComments {
         comments,
-        empty_lines,
+        node_kind,
     }
 }
 
@@ -558,8 +548,7 @@ pub(crate) fn empty_lines_before_trailing_comments<'a>(
 pub(crate) struct FormatEmptyLinesBeforeTrailingComments<'a> {
     /// The trailing comments of the node.
     comments: &'a [SourceComment],
-    /// The expected number of empty lines before the trailing comments.
-    empty_lines: u32,
+    node_kind: NodeKind,
 }
 
 impl Format<PyFormatContext<'_>> for FormatEmptyLinesBeforeTrailingComments<'_> {
@@ -569,9 +558,17 @@ impl Format<PyFormatContext<'_>> for FormatEmptyLinesBeforeTrailingComments<'_>
             .iter()
             .find(|comment| comment.line_position().is_own_line())
         {
+            // Black has different rules for stub vs. non-stub and top level vs. indented
+            let empty_lines = match (f.options().source_type(), f.context().node_level()) {
+                (PySourceType::Stub, NodeLevel::TopLevel(_)) => 1,
+                (PySourceType::Stub, _) => u32::from(self.node_kind == NodeKind::StmtClassDef),
+                (_, NodeLevel::TopLevel(_)) => 2,
+                (_, _) => 1,
+            };
+
             let actual = lines_before(comment.start(), f.context().source()).saturating_sub(1);
-            for _ in actual..self.empty_lines {
-                write!(f, [empty_line()])?;
+            for _ in actual..empty_lines {
+                empty_line().fmt(f)?;
             }
         }
         Ok(())
@@ -590,30 +587,16 @@ impl Format<PyFormatContext<'_>> for FormatEmptyLinesBeforeTrailingComments<'_>
 ///
 /// While `leading_comments` will preserve the existing empty line, this builder will insert an
 /// additional empty line before the comment.
-pub(crate) fn empty_lines_after_leading_comments<'a>(
-    f: &PyFormatter,
-    comments: &'a [SourceComment],
-) -> FormatEmptyLinesAfterLeadingComments<'a> {
-    // Black has different rules for stub vs. non-stub and top level vs. indented
-    let empty_lines = match (f.options().source_type(), f.context().node_level()) {
-        (PySourceType::Stub, NodeLevel::TopLevel(_)) => 1,
-        (PySourceType::Stub, _) => 0,
-        (_, NodeLevel::TopLevel(_)) => 2,
-        (_, _) => 1,
-    };
-
-    FormatEmptyLinesAfterLeadingComments {
-        comments,
-        empty_lines,
-    }
+pub(crate) fn empty_lines_after_leading_comments(
+    comments: &[SourceComment],
+) -> FormatEmptyLinesAfterLeadingComments {
+    FormatEmptyLinesAfterLeadingComments { comments }
 }
 
 #[derive(Copy, Clone, Debug)]
 pub(crate) struct FormatEmptyLinesAfterLeadingComments<'a> {
     /// The leading comments of the node.
     comments: &'a [SourceComment],
-    /// The expected number of empty lines after the leading comments.
-    empty_lines: u32,
 }
 
 impl Format<PyFormatContext<'_>> for FormatEmptyLinesAfterLeadingComments<'_> {
@@ -624,6 +607,14 @@ impl Format<PyFormatContext<'_>> for FormatEmptyLinesAfterLeadingComments<'_> {
             .rev()
             .find(|comment| comment.line_position().is_own_line())
         {
+            // Black has different rules for stub vs. non-stub and top level vs. indented
+            let empty_lines = match (f.options().source_type(), f.context().node_level()) {
+                (PySourceType::Stub, NodeLevel::TopLevel(_)) => 1,
+                (PySourceType::Stub, _) => 0,
+                (_, NodeLevel::TopLevel(_)) => 2,
+                (_, _) => 1,
+            };
+
             let actual = lines_after(comment.end(), f.context().source()).saturating_sub(1);
             // If there are no empty lines, keep the comment tight to the node.
             if actual == 0 {
@@ -632,12 +623,12 @@ impl Format<PyFormatContext<'_>> for FormatEmptyLinesAfterLeadingComments<'_> {
 
             // If there are more than enough empty lines already, `leading_comments` will
             // trim them as necessary.
-            if actual >= self.empty_lines {
+            if actual >= empty_lines {
                 return Ok(());
             }
 
-            for _ in actual..self.empty_lines {
-                write!(f, [empty_line()])?;
+            for _ in actual..empty_lines {
+                empty_line().fmt(f)?;
             }
         }
         Ok(())
